/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Security module.
 * 
 * This module handles the security mechanisms available on the running daemon.
 * See <waaad/security-api.h> for more information on the available functions, and details on how extensions can
 * register their functions.
 */
 
#ifndef _SECURITY_H
#define _SECURITY_H

/* Include the API definitions for the module */
#include <waaad/security-api.h>

/* For daemon's internal use only */
typedef struct _sec_item {
	int	 	  prio;	/* priority of this module -- only to build the list ordered */
	sec_module_t	 *sm;	/* pointer to the security module data */
	sec_mod_hdl_t	 *hdl;	/* pointer to the security module handle */
	struct _sec_item *next; /* The next module available to this peer with lesser priority, if any, or NULL. */
} _sec_item_t;


/* The following functions are called only from the daemon */

/*
 * FUNCTION:	sec_init
 *
 * PARAMETERS:
 *  -
 *
 * DESCRIPTION: 
 *  Initialize the module. 
 *
 * RETURN VALUE:
 *   0	: Application is now ready to use the module.
 *  !0  : An error occurred. 
 */
int sec_init ( void );

/*
 * FUNCTION:	sec_fini
 *
 * PARAMETERS:
 *  -
 *
 * DESCRIPTION: 
 *  Terminates the module. No sec_* function must be called after this one.
 *
 * RETURN VALUE:
 *   0 : module closed properly.
 *  !0 : an error occurred (we may ignore it)
 */
int sec_fini ( void );

/*
 * FUNCTION:	sec_module
 *
 * PARAMETERS:
 *  insecid: A value of Inband-Security-Id.
 *  module : upon success, the security module corresponding. The next field is always NULL.
 *
 * DESCRIPTION: 
 *  Get a reference to a security module from its ID.
 * An implicit call to sec_modlink is made on successful return.
 * The sec_freemodules must not be called on returned module list.
 *
 * RETURN VALUE:
 *   0 : Module found
 *  !0 : No such module
 */
int sec_module(uint32_t insecid, _sec_item_t *module);

/*
 * FUNCTION:	sec_getmodules
 *
 * PARAMETERS:
 *  diamid : diameter identity of the peer.
 *  sa     : Information about the location and port of the peer.
 *  modules: upon success, the security modules available for this peer.
 *
 * DESCRIPTION: 
 *  Get a list of security modules available to a peer, ordered by priority.
 * The list must be freed with sec_freemodules.
 * If the peer makes a successful CER/CEA exchange with one of the modules, it must call 
 * sec_modlink on its hdl; then call sec_modunlink when it does not need it anymore.
 *
 * RETURN VALUE:
 *   0 : List created properly
 *  !0 : an error occurred
 */
int sec_getmodules(char * diamid, sSA * sa, _sec_item_t **modules);

/*
 * FUNCTION:	sec_freemodules
 *
 * PARAMETERS:
 *  modules: The list of security modules to free.
 *
 * DESCRIPTION: 
 *  Release a list of modules returned by sec_getmodules.
 *
 * RETURN VALUE:
 *   0 : List freed properly
 *  !0 : an error occurred
 */
int sec_freemodules(_sec_item_t * modules);

/*
 * FUNCTION:	sec_modlink
 *
 * PARAMETERS:
 *  module: A security module to lock for use in a peer, after successful CER/CEA.
 *
 * DESCRIPTION: 
 *  Increment a refcount on a module.
 *
 * RETURN VALUE:
 *   0 : OK
 *  !0 : an error occurred
 */
int sec_modlink(sec_mod_hdl_t * module);

/*
 * FUNCTION:	sec_modunlink
 *
 * PARAMETERS:
 *  module: A security module to unlock after use in a peer.
 *
 * DESCRIPTION: 
 *  Decrement a refcount on a module.
 *
 * RETURN VALUE:
 *   0 : OK
 *  !0 : an error occurred
 */
int sec_modunlink(sec_mod_hdl_t * module);


#endif /* _SECURITY_H */
